# encoding: utf-8
#--
#   Copyright (C) 2011 Gitorious AS
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#++
require "json"
require "time"
require "htmlentities"
require "capillary/ref_collection"

module Capillary
  class Commit
    attr_accessor :id, :parent_ids, :committed_at, :committer_email, :seq_id
    attr_writer :message
    attr_reader :refs
    LOG_SEPARATOR = "§"

    def initialize
      @refs = RefCollection.new
    end

    # Creates an instance from Git output
    def self.parse(git_line)
      git_line = git_line.gsub(/^[^a-f0-9]+/, "")
      return nil if git_line == ""

      parts = git_line.force_encoding('utf-8').split(LOG_SEPARATOR)
      result = new
      result.id = parts[0]
      result.parent_ids = parts[1].split(" ")
      result.committed_at = Time.parse(parts[2])
      result.committer_email = parts[3]
      result.refs.parse(parts[4])
      result.message = parts[5]
      result
    end

    def to_hash
      { "id" => id,
        "parent_ids" => parent_ids,
        "committed_at" => committed_at,
        "committer_email" => committer_email,
        "refs" => refs.to_hash,
        "message" => message }
    end

    def to_json
      JSON.unparse({ "id" => id,
                     "parentIds" => parent_ids,
                     "committedAt" => committed_at,
                     "committerEmail" => committer_email,
                     "refs" => refs.to_hash,
                     "message" => message,
                     "seqId" => seq_id})
    end

    def message
      HTMLEntities.new.encode(@message)
    end
  end
end
