# encoding: utf-8
#--
#   Copyright (C) 2011 Gitorious AS
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#++
require "json"
require "capillary/commit"

module Capillary
  class LogParser
    attr_reader :branches

    def initialize
      @commit_count = 0
      @branches = []
      @seq_ids = {}
      @commit_positions = {}
    end

    def <<(commit)
      commit = Capillary::Commit.parse(commit) if String === commit
      return if commit.nil?

      seq_id = @commit_count
      @commit_count += 1

      commit.parent_ids.each do |parent_id|
        place_commit(CommitNode.new(seq_id, commit, parent_id))
      end

      place_commit(CommitNode.new(seq_id, commit, nil)) if commit.parent_ids.empty?
    end

    def to_json
      branches_json = branches.collect do |b|
        "[" + b.collect { |c| "#{c.to_json}" }.join(",") + "]"
      end

      "[#{branches_json.join(', ')}]"
    end

    private
    def place_commit(commit)
      col = 0

      placed = branches.inject(false) do |was_placed, branch|
        if !branch.last.closed? && branch.last.parent_id == commit.id
          save_commit_pos(col, commit)
          was_placed = true
        end

        col += 1
        was_placed
      end

      unless placed
        save_commit_pos(col, commit)
      end
    end

    def save_commit_pos(col, commit)
      branch = branches[col]
      prev_commit = branch && branch.last
      pos = prev_commit && @commit_positions[prev_commit.parent_id]

      if !pos
        branches << [] if branches.length <= col
        @commit_positions[commit.id] = [col, branches[col].length]
      end

      branches[col] << commit
    end
  end

  class CommitNode
    attr_reader :commit, :parent_id

    def initialize(seq_id, commit, parent_id)
      @commit = commit
      @commit.seq_id = seq_id
      @parent_id = parent_id
      @closed = false
    end

    def seq_id; commit.seq_id; end
    def id; commit.id; end
    def to_json; commit.to_json; end
    def closed?; @closed; end
    def close; @closed = true; end
  end
end
