# encoding: utf-8
#--
#   Copyright (C) 2011 Gitorious AS
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#++
require "json"

module Capillary
  class RefCollection
    def initialize
      @refs = {
        "heads" => [],
        "tags" => [],
        "remotes" => {}
      }
    end

    def self.parse(ref)
      refs = new
      refs.parse(ref)
      refs
    end

    def parse(ref)
      return if ref.nil? || ref == ""
      ref.gsub(/[\(\)]/,"").split(/,\s?/).each { |r| self << escape(r) }
    end

    def <<(full_ref)
      refs, type, *name = full_ref.strip.split("/")
      return if type.nil?
      type = type.gsub("-", "_")

      if type == "remotes"
        append_array_in_hash(@refs[type], name.shift, name)
      else
        append_array_in_hash(@refs, type, name)
      end
    end

    def to_hash
      @refs.dup
    end

    def to_json
      JSON.unparse(@refs)
    end

    def method_missing(method, *args, &block)
      key = method.to_s
      return super if !@refs.key?(key) || args.length != 0
      @refs[key]
    end

    private
    def escape(str)
      HTMLEntities.new.encode(str)
    end

    def append_array_in_hash(hash, key, names)
      hash[key] ||= []
      hash[key] << names.join("/")
    end
  end
end
